#!/bin/bash
#
#
# d-w-e-8-1
#
# priscilla3_installer.sh (14.1.2008)
#
# minimal priscilla3 installer (keep it simple!)
#

# debug
DEBUG=0

# impostazioni globali
umask 077
LOGFILE="${HOME}/priscilla3_install.log"
MYSQLPASSFILE="${HOME}/.mysqlpriscilla3"
PRISCILLA3SQLFILE="./build/priscilla3.sql"
CONFTEMPLATE="./build/config.template"
CONFFILE="adm/lib/conf.inc.php"
TEMPLATES="./build/installfiles/templates"
STYLE="./build/installfiles/style"
PIX="./build/installfiles/pix"

VERSION="0.1"
UPDATEMIRROR="http://sicomoro.plasticjumper.it/services/"



# registrazione su logfile
logreg () {
local message=$1

    echo  -e `date +'%Y-%m-%d %H:%M:%S'`" $message" >> ${LOGFILE}
}


# stampa warning
warnmessage () {
local message=$1

    logreg "warnmessage: ${message}"
    echo "priscilla3_installer: ${message}"
}

abortmessage () {
local message=$1

    logreg "abortmessage: ${message}"
    echo "priscilla3_installer: ${message}"
    exit 11
}


sethttpdownergroup () {

logreg "sethttpdownergroup"

        if [ -f "/etc/group" ]; then
                is_nobody=`cat /etc/group | grep nobody`
                if [ $? -eq 0 ]; then
                        owner="nobody"
                else
                        owner="www-data"
                fi
        fi
}


genpassword () {
#
# Random password generator for bash 2.x by Antek Sawicki <tenox@tenox.tc>,
# who generously gave permission to the document author to use it here.
#
# (Mendel Cooper)
#
MATRIX="0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
LENGTH=$1

    while [ "${n:=1}" -le "$LENGTH" ]
    do
        PASS="$PASS${MATRIX:$(($RANDOM%${#MATRIX})):1}"
        let n+=1
    done

echo $PASS
}

xquery () {

local user=$1
local dbname=$2
local query=$3
local password=$4

queryresult=""

if [ "${password}" = "" ]; then
    spassword="--defaults-extra-file=${MYSQLPASSFILE}"
    smaskedpassword=${spassword}
else
    smaskedpassword="-p <password>"
    spassword="-p${password}"
fi

logreg "mysql $smaskedpassword -s -u ${user} ${dbname} -e \"${query}\""
queryresult=`mysql ${spassword} -s -u  "${user}" "${dbname}" -e "${query}"`  2>> ${LOGFILE}
return $?
}


usage () {

    logreg "helpmessage"
    cat << EOF
Priscilla3 Installer V. ${VERSION}
Usage:
${0} [options]
-n      <sitename> (must be 6-16 chars long)
-m      <httppath> (site http path, eg.: http://sitename.domain)
-i      [install directory] (default: /var/www/vhost/)
-u      [mysql username] (default: <sitename>)
-p      [mysql user password] (default: auto generated)
-d      [mysql database name] (default: <sitename>)
-s      [superuser password] (default: auto generated)
-t      [mysql host name] (default: localhost)
-l      [language] (default: ita)
-k      [skin] (default: northsea)
-b      [backup directory] (default: /var/lib/pigei/priscilla3/<sitename>)
-o      [owner / group for installed files] (default: www-data || nobody)
-f      force a reinstall (Warning: it owerwrites existing database and installed site directory!)
-g      save mysql root password in \${HOME}/.mysqlpriscilla3
-e      <mail address> (send installation info to email address)
-h      print this help
EOF

}



# messaggi
MESSAGE_LOGFILE_BEGIN="Priscilla install"


# opzioni che hanno un default
forcereinstall=0
savemysqlpassword=0
sitename=""
httppath=""
mysqlrootpassword=""
backupdir=""
installdir="/var/www/vhost"
mysqlhostname="localhost"
lang="ita"
skin="northsea"
email=""

mysqlusername=""
superuserpassword=""
mysqldatabasename=""
mysqluserpassword=""


# determina l'owner dei file
sethttpdownergroup


# qui si inizia a lavorare

INSTALL_INFO="######### TERM: ${TERM} - OSTYPE = ${OSTYPE} - ARGS = $* - UID = ${UID} - EUID = ${EUID} USER = ${USER} - \
LOGNAME = ${LOGNAME} - SHELL = ${SHELL}"

echo -e "######### "`date +'%Y-%m-%d %H:%M:%S'`" ${MESSAGE_LOGFILE_BEGIN} version "${VERSION}" #########\n"${INSTALL_INFO} >> ${LOGFILE}

# vediamo se siamo stai invocati correttamente
if [ ! -f "build/priscilla3_installer.sh" ]; then
    abortmessage "You must invoke this command as './build/$(basename ${0})'"
fi

# warning se non root

if [ ! ${UID} -eq 0 ]; then
    warnmessage "You are not root"
fi


# parsing delle opzioni, in caso di mancanza, helpmessage

while getopts "hfgi:n:m:u:p:d:s:t:l:k:b:o:e:" OPTION
do
    case $OPTION in
        h)
             usage
             exit 0
             ;;
        f)
            forcereinstall=1
            ;;
        g)
            savemysqlpassword=1
            ;;
        i)
            installdir=$OPTARG
            ;;
        n)
            sitename=$OPTARG
            ;;
        m)
            httppath=$OPTARG
            ;;
        u)
            mysqlusername=$OPTARG
            ;;
        p)
            mysqluserpassword=$OPTARG
            ;;
        d)
            mysqldatabasename=$OPTARG
            ;;
        s)
            superuserpassword=$OPTARG
            ;;
        t)
            mysqlhostname=$OPTARG
            ;;
        l)
            lang=$OPTARG
            ;;
        k)
            skin=$OPTARG
            ;;
        b)
            backupdir=$OPTARG
            if [ -d ${backupdir} ]; then
                abortmessage "${backupdir} is not a valid backup directory"
            fi
            ;;
        o)
            owner=$OPTARG
            ;;
        e)
            email=$OPTARG
            ;;
        ?)
            usage
            exit
            ;;
     esac
done


# errore per le opzioni required
if [ "${sitename}" = "" ]; then
    abortmessage "<sitename> is required (See '${0} -h' for usage.)"
fi

if [ "${httppath}" = "" ]; then
    abortmessage "<httppath> is required (See '${0} -h' for usage.)"
fi

# test se nome site e' della lunghezza giusta
sitenamelen=`expr length ${sitename}`

if [ ${sitenamelen} -lt 6 -o ${sitenamelen} -gt 16 ]; then
    abortmessage "<sitename> must be 6-16 chars long"
fi

# test per caratteri consentiti nel nome sito

case ${sitename} in
    *[!a-z0-9_]*)
    abortmessage "<sitename> chars accepted: lowercase, 0-9, _"
    ;;
    *)
    ;;
esac

if [ "${backupdir}" = "" ]; then
    backupdir="/var/lib/priscilla3/${sitename}/"
fi


if [ "${mysqlusername}" = "" ]; then
    mysqlusername=$sitename
fi

if [ "${mysqldatabasename}" = "" ]; then
    mysqldatabasename=$sitename
fi

if [ "${mysqluserpassword}" = "" ]; then
    mysqluserpassword=$(genpassword 8)
fi

if [ "${superuserpassword}" = "" ]; then
    superuserpassword=$(genpassword 8)
fi


if [ ${DEBUG} -eq 1 ]; then
    echo -e "\n"
    echo "installdir: "${installdir}
    echo "sitename: "${sitename}
    echo "mysqlusername: "${mysqlusername}
    echo "mysqldatabasename: "${mysqldatabasename}
    echo "mysqluserpassword: "${mysqluserpassword}
    echo "superuserpassword: "${superuserpassword}
    echo "mysqlhostname: "${mysqlhostname}
    echo "lang: "${lang}
    echo "skin: "${skin}
    echo "backupdir: "${backupdir}
    echo "owner: "${owner}
fi


# creazione della directory di backup
mkdir -p ${backupdir}
if [ ! $? -eq 0 ]; then
    abortmessage "Could not create backup dir '${backupdir}'"
fi   

# chown backup dir
chown ${owner}:${owner} ${backupdir}
if [ ! $? -eq 0 ]; then
    abortmessage "Could not chown '${backupdir}' to ${owner}:${owner}"
fi   

# verifica dell'esistenza dell'install dir
if [ ! -d "${installdir}" ]; then
    abortmessage "Install dir '${installdir}' does not exists"
fi

# verifica se la directory di installazione e` gia` presente
if [ -d "${installdir}/${sitename}" ]; then
    if [ ${forcereinstall} -eq 0 ]; then
        abortmessage "sistename '${sitename}' already exists in '${installdir}'"
    else
        warnmessage "sitename '${sitename}' exists but a force reinstall is required by user"
    fi
else
    # creazione della dir di installazione
    mkdir "${installdir}/${sitename}/"
    if [ ! $? -eq 0 ]; then
        abortmessage "Could not create install dir '${installdir}/${dirname}/'"
    fi

    chmod 755 "${installdir}/${sitename}"
    if [ ! $? -eq 0 ]; then
        abortmessage "Could not chmod '${installdir}/${dirname}/' to 755"
    fi
fi   

# copia dei file di installazione
cp -a . ${installdir}/${sitename}/
if [ ! $? -eq 0 ]; then
    abortmessage "Could not copy install files to '${installdir}/${sitename}'"
fi   

# rimozione della directory build... (magari se non la si copiava)
rm -rf "${installdir}/${sitename}/build/"
if [ ! $? -eq 0 ]; then
    abortmessage "Could not remove build dir '${installdir}/${sitename}/build'"
fi   

if [ ! -f ${MYSQLPASSFILE} ]; then
    if [ "$mysqlrootpassword" = "" ]; then
        logreg "Reading mysql root password from standard input"
        echo -n "Please, enter mysql root password: "
        read -s mysqlrootpassword
    fi
fi

#creazione database
logreg "Creating database '${mysqldatabasename}'"
xquery "root" "" "CREATE DATABASE ${mysqldatabasename}" "${mysqlrootpassword}"
if [ ! $? -eq 0 ]; then

    # se il tabase esiste da` un errore, almeno che non sia stato specificato un force reinstall
    if [ ${forcereinstall} -eq 0 ]; then
        abortmessage "Could not create database '${sitename}'"
    else
        warnmessage "databasenane '${mysqldatabasename}' exists but a force reinstall was required by user" 
    fi
fi

# importazione sql
if [ "${mysqlrootpassword}" = "" ]; then
    spassword="--defaults-extra-file=${MYSQLPASSFILE}"
else
    spassword="-p${mysqlrootpassword}"
fi

logreg "Importing sql file '${PRISCILLA3SQLFILE}'"
mysql ${spassword} -s -u "root" "${mysqldatabasename}" < ${PRISCILLA3SQLFILE} 2>> ${LOGFILE}
if [ ! $? -eq 0 ]; then
    abortmessage "Importing priscilla3 sql file failed"
fi


# vediamo se e` richiesto un salvataggio password

if [ ! -f ${MYSQLPASSFILE} ]; then
    if [ $savemysqlpassword -eq 1 ]; then
        cat << EOF > ${MYSQLPASSFILE}
[mysql]
password='${mysqlrootpassword}'
[mysqldump]
password='${mysqlrootpassword}'
EOF
    fi
else
    if [ $savemysqlpassword -eq 1 ]; then
        warnmessage "A config file was required but '${MYSQLPASSFILE}' already exists"
    fi
fi

# assegnazione privilegi
logreg "GRANT ALL PRIVILEGES"
xquery "root" "" "GRANT ALL PRIVILEGES ON ${mysqldatabasename}.* TO '${mysqlusername}'@'${mysqlhostname}' IDENTIFIED BY '${mysqluserpassword}';" "${mysqlrootpassword}"
if [ ! $? -eq 0 ]; then
    abortmessage "Could not grant privileges on ${mysqldatabasename}.* to '${mysqlusername}'@'${mysqlhostname}'"
fi

# flush privilegi
logreg "FLUSH PRIVILEGES"
xquery "root" "" "FLUSH PRIVILEGES" "${mysqlrootpassword}"
if [ ! $? -eq 0 ]; then
    abortmessage "Could flush privileges"
fi

sha1sum_password=$(echo -n ${superuserpassword} | sha1sum)

# creazione dell'utente superuser
logreg "Creating superuser"
xquery "root" "${mysqldatabasename}" "INSERT INTO \`user\` (\`livello\`, \`adm\`, \`username\`, \`password\`,\`nome\`,\`sospeso\`,\`lingua\`,\`skin\`,\`editor\`,\`htmlLevel\`) VALUES ('0', 'si', 'superuser', '${sha1sum_password}', 'superuser', 'no', '${lang}', '${skin}', 'e', 'high')" "${mysqlrootpassword}"
if [ ! $? -eq 0 ]; then
    abortmessage "Could not create superuser"
fi


# setting password crittate
#logreg "Setting crypted password option"
#xquery "root" "${mysqldatabasename}" "UPDATE \`security_prefs\` SET \`security_crypted_passwd\` = '1' WHERE \`id\` = 1" "${mysqlrootpassword}"
#if [ ! $? -eq 0 ]; then
#    abortmessage "Could not set crypted password option"
#fi

# copia dei templates
cp ${TEMPLATES}/* "${installdir}/${sitename}/templates/"
if [ ! $? -eq 0 ]; then
    abortmessage "Could not copy templates files from ${TEMPLATES} to ${installdir}/${sitename}/templates/"
fi

# chmod templates
chmod 644 ${installdir}/${sitename}/templates/*
if [ ! $? -eq 0 ]; then
    abortmessage "Could not chmod '"${installdir}/${sitename}/templates/*"' to 644"
fi

# copia dei fogli di stile
cp ${STYLE}/* "${installdir}/${sitename}/style/"
if [ ! $? -eq 0 ]; then
    abortmessage "Could not copy style files from ${STYLE} to ${installdir}/${sitename}/style/"
fi

# chmod syles
chmod 644 ${installdir}/${sitename}/style/*

if [ ! $? -eq 0 ]; then
    abortmessage "Could not chmod '"${installdir}/${sitename}/style/*"' to 644"
fi

# copia delle immagini
cp -r ${PIX}/* "${installdir}/${sitename}/pix/"
if [ ! $? -eq 0 ]; then
    abortmessage "Could not copy pix files from ${PIX} to ${installdir}/${sitename}/pix/"
fi

# chmod pix
chmod -R 644 ${installdir}/${sitename}/pix/*

if [ ! $? -eq 0 ]; then
    abortmessage "Could not chmod '"${installdir}/${sitename}/pix/*"' to 644"
fi


# generazione del file di configurazione
cat "${CONFTEMPLATE}" | sed "s:%opt_TESTING%:0:" | sed "s:%dbtype%:mysql:" | sed "s:%hostname%:${mysqlhostname}:" | sed "s:%dbuser%:${mysqlusername}:" | sed "s:%dbpasswd%:${mysqluserpassword}:" | sed "s:%dbname%:${mysqldatabasename}:" | sed 's:%opt_passwcrpt%:0:' | sed "s:%smtpserver%:localhost:" | sed "s:%smtpuser%::" | sed "s:%smtppassw%::" | sed "s:%smtprealm%::" | sed "s:%opt_lang%:${lang}:" | sed "s:%define_LANG%:${lang}:" | sed "s:%opt_skin%:${skin}:" | sed "s|%opt_priscilla_update_mirror%|${UPDATEMIRROR}|" | sed "s:%opt_priscilla_local_path%:${installdir}/${sitename}/:" | sed "s:%define_PRISCILLA_LOCAL_PATH%:${installdir}/${sitename}/:" | sed "s:%opt_priscilla_backup_dir%:${backupdir}:" | sed "s|%opt_priscilla_server_path%|${httppath}/|" | sed "s|%define_PRISCILLA_SERVER_PATH%|${httppath}/|" > "${installdir}/${sitename}/${CONFFILE}"

if [ ! $? -eq 0 ]; then
    abortmessage "Could not create config file '${installdir}/${sitename}/${CONFFILE}'"
fi


#generazione della documentazione
cd "${installdir}/${sitename}"
bash 1> /dev/null generadoc.sh

# chown del sito
chown -R ${owner}:${owner} "${installdir}/${sitename}/"
if [ ! $? -eq 0 ]; then
    abortmessage "Could not chown '${installdir}/${sitename}/' to ${owner}:${owner}"
fi

echo -e "\nsuperuser: ${superuserpassword}\n"


# se richiesto invia informazioni dell'installazione all'indirizzo email specificato

if [ "$email" !=  "" ]; then
mail -s "Priscilla3 new site: ${sitename} (${httppath})" ${email} << EOF

superuser: ${superuserpassword}
httppath: ${httppath}
localpath: ${mysqlhostname}
databasename: ${mysqldatabasename}
mysqluser: ${mysqlusername}
mysqlhostname: ${mysqlhostname}
lang: ${lang}
skin: ${skin}

EOF

if [ ! $? -eq 0 ]; then
    abortmessage "Could not send the info email to '${email}'"
fi

fi


exit 0
