#!/bin/bash

# priscilla_pack.sh
# crea un archivio di installazione partendo dalla priscilla attuale

LOGFILE="${HOME}/priscilla_pack.log"
MYSQLPASSFILE="${HOME}/.mysqlpassall"

CONFFILE="adm/lib/conf.inc.php"
CONFTEMPLATE="./build/config.template"

# registrazione su logfile
logreg () {
local message=$1

    echo  -e `date +'%Y-%m-%d %H:%M:%S'`" $message" >> ${LOGFILE}
}

# stampa warning
warnmessage () {
local message=$1

    logreg "warnmessage: ${message}"
    echo "warning: ${message}"
}

abortmessage () {
local message=$1

    logreg "abortmessage: ${message}"
    echo "error: ${message}"
    exit 11
}

# vediamo se siamo stai invocati correttamente
if [ ! -f "build/priscilla_pack.sh" ]; then
    abortmessage "You must invoke this command as './build/$(basename ${0})'"
fi

# warning se non root
if [ ! ${UID} -eq 0 ]; then
    abortmessage "You must be root"
fi

# pulisco la dir pack se esiste già
if [ -d build/pack/ ]; then
    echo -n "Pack directory already exists. Delete and continue? [yes/NO] "
    read answer
    
    if [ "$answer" != "yes" ] ; then
        abortmessage "Pack aborted"
    else
        rm -rf build/pack/*
    fi
else
    echo "Creating pack directory...";
    mkdir build/pack
fi


# prendo il nome del db dal file di configurazione
originaldbname=$(php -r "require \"${CONFFILE}\"; echo \$dbname;")
if [ "${originaldbname}" = "" ]; then
    abortmessage "Original dbname not found in ${CONFFILE}"
fi

echo "Packing database..."
mysqldump --defaults-extra-file=${MYSQLPASSFILE} --opt -h localhost -u root --quote-names --add-drop-table  ${originaldbname} >> build/pack/sql.pack

# generazione del file di configurazione
echo "Conf generation..."
cat "${CONFTEMPLATE}" | sed "s:%opt_TESTING%:0:" | sed "s:%dbtype%:mysql:" | sed "s:%hostname%:localhost:" | sed 's:%opt_passwcrpt%:0:' | sed "s:%smtpserver%:localhost:" | sed "s:%smtpuser%::" | sed "s:%smtppassw%::" | sed "s:%smtprealm%::" | sed "s:%opt_lang%:ita:" | sed "s:%define_LANG%:ita:" | sed "s:%opt_skin%:sandworm:" | sed "s|%opt_priscilla_update_mirror%|sicomoro.plasticjumper.it/services|" > build/pack/conf.pack

originaldbname="priscilla_riuso"

echo "Packing files..."
CURRDIRNAME=$(basename `pwd`)
cd ..
tar -czf ${HOME}/${originaldbname}_pack.tar.gz ${CURRDIRNAME}
cd ${CURRDIRNAME}

echo -e "Pack created in ${HOME}/${originaldbname}_pack.tar.gz\n\nHave a nice day."
logreg "Pack created in ${HOME}/${originaldbname}_pack.tar.gz"
exit 0
